# $Id: qtedecl.rb,v 1.2 2006/07/14 09:15:26 noir Exp $
# qtedecl - Add a declarational syntax to Ruby/Qte UI design
# Copyright (C) 2006 Hiroyuki KUROSAKI <noir (at) st . rim . or . jp>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

=begin
= Description

This library provides a declarational syntax to Ruby/Qte UI
design coding.

By using this definitions, we can avoid such a boring coding like:

  bg = QHButtonGroup.new(self)
  b1 = QPushButton.new(QString::fromUtf8('Cancel'), bg)
  b1.setFixedSize(b1.sizeHint)
  b2 = QPushButton.new(QString::fromUtf8('OK'), bg)
  b2.setFixedSize(b2.sizeHint)
  ...

Instead:

  h_button_group {
    push_button 'Cancel'
    push_button 'OK'
  }

=end

require 'qte'
require 'qpe'

module Qte

# Utility methods.

  def qs(rstr)
    QString::fromUtf8(rstr.to_s)
  end

  def u8(qstr)
    qstr.utf8().to_str
  end


  def q_widget(klass, *args)
    klass.new(*args)
  end

  def uncapitalize(str)
    str.scan(/([A-Z])([^A-Z]*)/).map {|h, rest| h.downcase + rest} .
        join('_')
  end

  def capitalize2(str)
    str[0, 1].upcase + str[1...str.size]
  end

  module_function :uncapitalize, :capitalize2

  def qd_context(parent = nil)
    parent || (@qd_widget_stack && @qd_widget_stack.last) || self
  end

  def qd_nest(widget)
    @qd_widget_stack = [] unless @qd_widget_stack
    @qd_widget_stack.push(widget)
    yield
    @qd_widget_stack.pop
    widget
  end

# call-seq:
#   label(text, parent = nil)
#   push_button(text, parent = nil)
#   radio_button(text, parent = nil)
#   check_box(text, parent = nil)
#
# Declare small widgets. Sizes are adjusted to fit the text.
# When parent is nil,
#   1. If used in the widget block, it is owned by the block widget.
#   2. If not, owned by self.

  %w(Label PushButton RadioButton CheckBox).each do |klass_name|
    module_eval <<EOF
      def #{uncapitalize(klass_name)}(text, parent = nil, *args)
        w = q_widget(Q#{klass_name}, qs(text), qd_context(parent), *args)
        w.setFixedSize(w.sizeHint)
        w
      end
EOF
  end

# call-seq:
#   line_edit(parent = nil)
#   multi_line_edit(parent = nil)
#   combo_box(parent = nil)
#   tab_widget(parent = nil)
#
# Declare widgets which don't require size adjustment.

  %w(LineEdit MultiLineEdit ComboBox TabWidget).each do |klass_name|
    module_eval <<EOF
      def #{uncapitalize(klass_name)}(parent = nil, *args)
        q_widget(Q#{klass_name}, qd_context(parent), *args)
      end
EOF
  end

# call-seq:
#   h_box(parent = nil) {|box| ... }
#   v_box(parent = nil) {|box| ... }
#   h_button_group(parent = nil) {|group| ...}
#   v_button_group(parent = nil) {|group| ...}
#
# Declare horizontal/vertical boxes, etc.
# When parent is nil, the widget is owned by self.
# Provide a block to insert child widgets.
# Indentation improves readability.
# 

  %w(HBox VBox PEToolBar).each do |klass_name|
    module_eval <<EOF
      def #{uncapitalize(klass_name)}(parent = nil, *args)
        box = Q#{klass_name}.new(qd_context(parent), *args)
        qd_nest(box) {
          yield box
        } if block_given?
        box
      end
EOF
  end

# adjust size
  %w(HButtonGroup VButtonGroup).each do |klass_name|
    module_eval <<EOF
      def #{uncapitalize(klass_name)}(parent = nil, *args)
        button_group = Q#{klass_name}.new(qd_context(parent), *args)
        qd_nest(button_group) {
          yield button_group
        } if block_given?
        button_group.setFixedSize(button_group.sizeHint)
        button_group
      end
EOF
  end

# Call-seq:
#   grid(n, direction, parent = nil, *args) {|gr| ... }

  def grid(n, direction, parent = nil, *args)
    gr = QGrid.new(n, direction, qd_context(parent), *args)
    qd_nest(gr) {
      yield gr
    } if block_given?
    gr
  end

# Call-seq:
#   scroll_view(parent = nil) {|sv| ...}
#
# Notice: Widgets inside block are added to the viewpoint by default, but
# the block argument is the scroll view itself.

  def scroll_view(parent = nil, *args)
    sv = QScrollView.new(qd_context(parent), *args)
    qd_nest(sv.viewport) {
      yield sv
    } if block_given?
    sv
  end

# = Avoiding using ugly setter methods
#
# You can call setter methods by assigning a value to the property.
#
# Example:
#
#   edit = multi_line_edit
#   edit.text = qs('foo')

  def method_missing(name, *args)
    setter = (/^([^=]+)=$/ =~ name.to_s) && ('set' + capitalize2($1))
    if setter && self.methods.include?(setter) then
      self.send(setter, *args)
    else
      Kernel.raise
    end
  end
end
